"""
Contains the various modules' call schemas. These
files are joined together in ../call_schema.py
"""

from .cloudagent_schema import CLOUDAGENT_SCHEMA
from .cs_schema import CS_SCHEMA
from .gav_schema import GAV_SCHEMA
from .pm_schema import PM_SCHEMA
from .totalcloud_schema import TOTALCLOUD_SCHEMA
from .vmdr_schema import VMDR_SCHEMA
from .was_schema import WAS_SCHEMA
from .certview_schema import CERTVIEW_SCHEMA
